unit fFolderOpt;

interface

uses
  Windows, SysUtils, Classes, Controls, Forms, graphics, JvBaseDlg,
  JvBrowseFolder, ImgList, Dialogs, ExtDlgs, ComCtrls, StdCtrls, Buttons,
  JvComponent;

type
  TFrmFolder = class(TForm)
    BtnOK: TButton;
    BtnCancel: TButton;
    LblFolder: TLabel;
    LblFol: TLabel;
    OpenPictureDialog: TOpenPictureDialog;
    IconList: TImageList;
    PageFol: TPageControl;
    TabSheet1: TTabSheet;
    GrpMirror: TGroupBox;
    lblDir: TLabel;
    lblEmu: TLabel;
    lblFilter: TLabel;
    BtnAddFilter: TSpeedButton;
    BtnFilterMinus: TSpeedButton;
    Label1: TLabel;
    LstFilter: TListBox;
    cmbEmu: TComboBox;
    ChkMirror: TCheckBox;
    GrpMedia: TGroupBox;
    lblPath: TLabel;
    BtnAdd: TSpeedButton;
    BtnMinus: TSpeedButton;
    LstPaths: TListBox;
    TabSheet3: TTabSheet;
    TxtRealIconDir: TEdit;
    BtnRealIconDir: TButton;
    lblMAMElink: TLabel;
    ChkLargeIcons: TCheckBox;
    ChkRealIcons: TCheckBox;
    jvBrowse: TJvBrowseForFolderDialog;
    LstDirMir: TListBox;
    BtnAddMir: TSpeedButton;
    BtnDelMir: TSpeedButton;
    GrpBackground: TGroupBox;
    LblBKPath: TLabel;
    ChkBK: TCheckBox;
    TxtBKPath: TEdit;
    BtnBKDir: TButton;
    GrpIcon: TGroupBox;
    lblIcon: TLabel;
    ChkIcon: TCheckBox;
    CmbIcon: TComboBoxEx;
    procedure BtnMinusClick(Sender: TObject);
    procedure BtnAddClick(Sender: TObject);
    procedure BtnBKDirClick(Sender: TObject);
    procedure BtnCancelClick(Sender: TObject);
    procedure BtnOKClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ChkBKClick(Sender: TObject);
    procedure ChkIconClick(Sender: TObject);
    procedure ChkMirrorClick(Sender: TObject);
    procedure BtnAddFilterClick(Sender: TObject);
    procedure BtnFilterMinusClick(Sender: TObject);
    procedure BtnRealIconDirClick(Sender: TObject);
    procedure lblMAMElinkClick(Sender: TObject);
    procedure BtnAddMirClick(Sender: TObject);
    procedure BtnDelMirClick(Sender: TObject);
  private
    procedure Init;
    { Private declarations }

  public
    { Public declarations }
    SelFol : String;
  end;

implementation

uses JCLStrings, IniFiles, FindFile, ShellAPI, fMain, uJUtilities,
  uEmuList, uQPConst;

{$R *.dfm}

Procedure TFrmFolder.Init();
Var
  I : Integer;
  temp1, TempLbl : String;
  Image : TIcon;
  Find : ThhFindFile;
  Ini : TMemIniFile;
  List : TMemoryStream;
  Dirs : TStringList;
begin

  {-----------------------------------------------}
  {Set up the Folder Name at the top of the form}

  Temp1 := SelFol;
  Temp1 := Trim(Temp1);
  Temp1 := ExcludeTrailingPathDelimiter(Temp1);

  For i := Length(Temp1)-1 Downto 0 do
  begin
    if Temp1[i] = '\' then break;
  end;

  TempLbl := Copy(Temp1, i+1, Length(Temp1)-1);

  LblFol.Caption := TempLbl;

  {------------------------------------------------}
  { Add all available emulators to mirror combo box}
  MainFrm.EmuList.EmusToStrings(CmbEmu.Items, cfAllEmus);

  {------------------------------------------------}
  {Set up the combo box with the icons}

  Image := nil;
  Find := ThhFindFile.Create(Self);
  try
    Image := TIcon.Create;
    Find.Directory := MainFrm.Settings.QPDirectory + 'icons\';
    find.Filter := '*.ico';
    Find.Recurse := False;

    Find.Execute;

    Dirs := TStringList.Create;
    try

      For i := 0 to Find.TotalFile-1 do
      begin

        try
          //if this fails then the specified file is not really an icon.
          Image.LoadFromFile(Find.Files[i]);
          //if it didnt fail then we can add it to the icon list and combo
          IconList.AddIcon(Image);
          CmbIcon.ItemsEx.AddItem(ExtractFileName(Find.files[i]),i,i,i,0,nil)
        //if it failed, add it to the failure stringlist
        except
          Dirs.Add(ExtractFileName(Find.Files[i]));
        end;

      end;

      If Dirs.Count > 0 then
        MessageDlg(QP_FOLOPTFRM_BAD_ICONS + Dirs.GetText, mtError, [mbOK], 0);

    Finally
      FreeAndNil(Dirs);
    end;

  finally
    FreeAndNil(Find);
    FreeAndNil(Image);
  end;

  {------------------------------------------------}
  {Set up the options if the settings have been written}

  Ini := TMemIniFile.Create(SelFol + 'folders.ini');
  Try

      //Mirroring Options
      ChkMirror.Checked := Ini.ReadBool('Mirror', 'ChkMirror', False);

      Dirs := TStringList.Create;
      try
        StrTokenToStrings(Ini.ReadString('Mirror', 'TxtDir', ''),';', Dirs);
        For i := 0 to Dirs.Count-1 do
          LstDirMir.Items.Add(Dirs.Strings[i])
      Finally
        FreeAndNil(Dirs);
      end;
      //TxtDir.Text := Ini.ReadString('Mirror', 'TxtDir', '');

      CmbEmu.ItemIndex := CmbEmu.Items.IndexOf(Ini.ReadString('Mirror','CmbEmu', ''));

      List := TMemoryStream.Create;
      try
        Ini.ReadBinaryStream('Mirror','LstFilter',List);
        LstFilter.Items.LoadFromStream(List);

      finally
        FreeAndNil(List);
      end;
      
      If LstFilter.Items.Count = 0 then
        LstFilter.Items.Add('*.zip');

      BtnAddfilter.Enabled := ChkMirror.Checked;
      BtnFilterMinus.Enabled := ChkMirror.Checked;
      CmbEmu.Enabled := ChkMirror.Checked;
      lblDir.Enabled := ChkMirror.Checked;
      lblEmu.Enabled := ChkMirror.Checked;
      lblFilter.Enabled := ChkMirror.Checked;
      LstFilter.Enabled := ChkMirror.Checked;
      LstDirMir.Enabled := ChkMirror.Checked;
      BtnAddMir.Enabled := ChkMirror.Checked;
      BtnDelMir.Enabled := ChkMirror.Checked;

      //Rom Listing Background
      ChkBk.Checked := Ini.ReadBool('BkGround', 'ChkBk', False);
      TxtBKPath.Text := Ini.ReadString('BkGround', 'TxtBKPath', '');

      TxtBKPath.Enabled := ChkBk.Checked;
      LblBKPath.Enabled := ChkBK.Checked;
      BtnBKDir.Enabled := ChkBk.Checked;

      //Folder List Icon
      ChkIcon.Checked := Ini.ReadBool('Icon', 'ChkIcon', False);
      CmbIcon.ItemIndex := CmbIcon.Items.IndexOf(Ini.ReadString('Icon','CmbIcon', ''));
      CmbIcon.Enabled := ChkIcon.Checked;
      LblIcon.Enabled := ChkIcon.Checked;

      //Media Paths
      List := TMemoryStream.Create;
      try
        Ini.ReadBinaryStream('MediaPaths','LstPaths',List);
        LstPaths.Items.LoadFromStream(List);

      finally
        FreeAndNil(List);
      end;

      // Real Icons settings
      ChkRealIcons.Checked := Ini.ReadBool('RealIcon', 'ChkRealIcons', false);
      TxtRealIconDir.Text := Ini.ReadString('RealIcon', 'Directory', '');
      ChkLargeIcons.Checked := Ini.ReadBool('RealIcon', 'ChkLargeIcons', false);
  Finally
    FreeAndNil(Ini);
  end;

end;

{-----------------------------------------------------------------------------}

procedure TFrmFolder.BtnMinusClick(Sender: TObject);
begin
  LstPaths.Items.Delete(LstPaths.itemindex);
end;

{-----------------------------------------------------------------------------}

procedure TFrmFolder.BtnAddClick(Sender: TObject);
begin
  if jvBrowse.execute then
    if jvBrowse.Directory <> '' then
      LstPaths.Items.Add(jvBrowse.Directory);
end;

{-----------------------------------------------------------------------------}

procedure TFrmFolder.BtnBKDirClick(Sender: TObject);
begin
  if OpenPictureDialog.Execute = true then
    if (OpenPictureDialog.FileName <> '') AND (fileexists(OpenPictureDialog.FileName) = true) then
       TxtBKPath.Text := OpenPictureDialog.FileName;
end;

{-----------------------------------------------------------------------------}

procedure TFrmFolder.BtnCancelClick(Sender: TObject);
begin
  ModalResult := MrCancel;
end;

{-----------------------------------------------------------------------------}

procedure TFrmFolder.BtnOKClick(Sender: TObject);
Var
  Ini : TMemIniFile;
  List : TMemoryStream;
  DirOutput : String;
  i : Integer;
begin

  Ini := TMemInifile.create(SelFol + 'folders.ini');
  Try

    //Mirroring Options
    Ini.WriteBool('Mirror', 'ChkMirror', ChkMirror.Checked);
    DirOutput := '';
    For i := 0 to LstDirMir.Count-1 do
      If DirectoryExists(LstDirMir.Items.Strings[i]) then
        DirOutput := DirOutput + LstDirMir.Items.Strings[i] + ';';

    Ini.WriteString('Mirror', 'TxtDir', DirOutput);

    if CmbEmu.ItemIndex <>-1 then
      Ini.WriteString('Mirror', 'CmbEmu', CmbEmu.Items.Strings[CmbEmu.ItemIndex]);

    //Media Paths
    List := TMemoryStream.Create;
    try

      LstFilter.Items.SaveToStream(List);
      List.Position := 0;
      Ini.WriteBinaryStream('Mirror','LstFilter', List);

    finally
      FreeAndNil(List);
    end;

    // Real Icon Support
    Ini.WriteBool('RealIcon', 'ChkRealIcons', ChkRealIcons.Checked);
    If DirectoryExists(TxtRealIconDir.Text) then
      Ini.WriteString('RealIcon', 'Directory', TxtRealIconDir.Text);
    Ini.WriteBool('RealIcon', 'ChkLargeIcons', ChkLargeIcons.Checked);

    //Background options
    Ini.WriteBool('BkGround', 'ChkBk', ChkBK.Checked);
    Ini.WriteString('BkGround', 'TxtBKPath', TxtBKPath.Text);

    //Folder List Icon
    Ini.WriteBool('Icon', 'ChkIcon', ChkIcon.Checked);
    if CmbIcon.ItemIndex <>-1 then
      Ini.WriteString('Icon', 'CmbIcon', CmbIcon.Items.Strings[CmbIcon.ItemIndex]);

    //Media Paths
    List := TMemoryStream.Create;
    try

      LstPaths.Items.SaveToStream(List);
      List.Position := 0;
      Ini.WriteBinaryStream('MediaPaths','LstPaths', List);

    finally
      FreeAndNil(List);
    end;

    if MainFrm.Settings.AllowWrite then
      Ini.UpdateFile;
    
  Finally
    FreeAndNil(Ini);
  end;

  ModalResult := MrOK;
end;

{-----------------------------------------------------------------------------}

procedure TFrmFolder.FormShow(Sender: TObject);
begin
  PageFol.ActivePageIndex := 0;
  Init();
  if DirectoryExists(MainFrm.Settings.QPDirectory + 'images') then
    OpenPictureDialog.InitialDir := MainFrm.Settings.QPDirectory + 'images';
end;

{-----------------------------------------------------------------------------}

procedure TFrmFolder.ChkBKClick(Sender: TObject);
begin
    TxtBKPath.Enabled := ChkBk.Checked;
    LblBKPath.Enabled := ChkBK.Checked;
    BtnBKDir.Enabled := ChkBk.Checked;
end;

{-----------------------------------------------------------------------------}

procedure TFrmFolder.ChkIconClick(Sender: TObject);
begin
  CmbIcon.Enabled := ChkIcon.Checked;
  LblIcon.Enabled := ChkIcon.Checked;
end;

{-----------------------------------------------------------------------------}

procedure TFrmFolder.ChkMirrorClick(Sender: TObject);
begin
  BtnAddfilter.Enabled := ChkMirror.Checked;
  BtnFilterMinus.Enabled := ChkMirror.Checked;
  CmbEmu.Enabled := ChkMirror.Checked;
  lblDir.Enabled := ChkMirror.Checked;
  lblEmu.Enabled := ChkMirror.Checked;
  lblFilter.Enabled := ChkMirror.Checked;
  LstFilter.Enabled := ChkMirror.Checked;
  LstDirMir.Enabled := ChkMirror.Checked;
  BtnAddMir.Enabled := ChkMirror.Checked;
  BtnDelMir.Enabled := ChkMirror.Checked;
end;

{-----------------------------------------------------------------------------}

procedure TFrmFolder.BtnAddFilterClick(Sender: TObject);
begin
  LstFilter.items.Add(inputBox(QP_FOLOPTFRM_FILE_EXT,QP_FOLOPTFRM_FILE_EXT,''));
end;

{-----------------------------------------------------------------------------}

procedure TFrmFolder.BtnFilterMinusClick(Sender: TObject);
begin
  LstFilter.Items.Delete(LstFilter.itemindex);
end;

{-----------------------------------------------------------------------------}

procedure TFrmFolder.BtnRealIconDirClick(Sender: TObject);
begin
  if jvBrowse.execute then
    if (jvBrowse.Directory <> '') and (directoryExists(jvBrowse.Directory)) then
      TxtRealIconDir.Text := jvBrowse.Directory;
end;

{-----------------------------------------------------------------------------}

procedure TFrmFolder.lblMAMElinkClick(Sender: TObject);
begin
  ShellExecute(Handle, 'open', PChar('http://www.mameworld.net/icons/'), '', '', sw_Show);
end;

{-----------------------------------------------------------------------------}

procedure TFrmFolder.BtnAddMirClick(Sender: TObject);
begin
  if jvBrowse.execute then
    if jvBrowse.Directory <> '' then
      LstDirMir.Items.Add(jvBrowse.Directory);
end;

procedure TFrmFolder.BtnDelMirClick(Sender: TObject);
begin
  LstDirMir.Items.Delete(LstDirMir.itemindex);
end;

end.
