unit fMameMerge;

interface

uses
  Windows, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, JvBaseDlg, JvSelectDirectory, JvBrowseFolder, Buttons,
  uRomList, JvComponent;

type
  TFrmMAMEMerged = class(TForm)
    CmbMame: TComboBox;
    lblMAME: TLabel;
    BtnOK: TButton;
    BtnCancel: TButton;
    Label1: TLabel;
    lblDir: TLabel;
    jvBrowse: TJvBrowseForFolderDialog;
    ChkCatVer: TCheckBox;
    ListDirs: TListBox;
    BtnAdd: TSpeedButton;
    BtnMinus: TSpeedButton;
    RadClear: TRadioButton;
    RadRefreshList: TRadioButton;
    procedure FormShow(Sender: TObject);
    procedure BtnOKClick(Sender: TObject);
    procedure BtnAddClick(Sender: TObject);
    procedure BtnMinusClick(Sender: TObject);
  private
    { Private declarations }
    Function ROMIsIn(Clone, Path : String; Searchlist : TQPROMlist): Integer;
  public
    { Public declarations }
  end;

implementation

uses fMain, uJUtilities, StrUtils, JCLstrings, IniFiles, uRom, uEmuList, uQPConst;

{$R *.dfm}

procedure TFrmMAMEMerged.FormShow(Sender: TObject);
var
  Ini : TMemIniFile;
  Dirs : TStringList;
  I : Integer;
begin
  MainFrm.EmuList.EmusToStrings(CmbMame.Items, cfMame);

  Ini := TMemIniFile.Create(MainFrm.Settings.SettingsFile);
  Try

    //Load in the Directories.
    Dirs := TStringList.Create;
    try
      StrTokenToStrings(Ini.ReadString('MAMEScan', 'Dirs', ''),';', Dirs);
      For i := 0 to Dirs.Count-1 do
        ListDirs.Items.Add(Dirs.Strings[i])
    Finally
      FreeAndNil(Dirs);
    end;

    CmbMAME.ItemIndex := CmbMAME.Items.IndexOf(Ini.ReadString('MAMEScan', 'MAME', ''));
    ChkCatVer.Checked := Ini.ReadBool('MAMEScan', 'CatVer', false);
    If Ini.ReadInteger('MAMEScan', 'ScanType', 1) = 1 then
      RadClear.Checked := true
    else
      RadRefreshList.Checked := true;

  Finally
    FreeAndNil(Ini);
  end;

end;

procedure TFrmMAMEMerged.BtnOKClick(Sender: TObject);
var
  CommandLine, Name, DirOutput : String;
  Flist : TStringList;
  I, Dirs, FIndex : Integer;
  Ini : TMemIniFile;
  TmpROMs : TQPROMlist;
begin

  If (CmbMame.ItemIndex = -1) or (ListDirs.Count = 0) then
  begin
    MessageDlg(QP_MAMEM_BAD_DATA, mtError, [mbOK], 0);
    exit;
  end;

  //check that there isnt a vset file.  If there is, delete it.
  If FileExists(MainFrm.Settings.SettingsDir + 'vset.txt') then
    DeleteFile(MainFrm.Settings.SettingsDir + 'vset.txt');

  //first stage is to scan all the directories and get a full
  //listing of all ROMS we need to add.
  TmpROMs := TQPROMlist.Create(MainFrm.Settings.AllowWrite);
  try
    For Dirs := 0 to ListDirs.Count-1 do
    begin
      //set up the commandline for MAME to process the directory
      commandline := extractshortpathname(MainFrm.EmuList[MainFrm.EmuList.IndexOfName(CmbMame.Text)].Path);
      commandline := commandline + ' -verifysets  -rompath ' + ExtractShortPathName(ListDirs.Items[Dirs]) + ' > ' + extractshortpathname(MainFrm.Settings.SettingsDir) + 'vset.txt';
      case Win32Platform of
        0,1: RunProcess('command.com /c '+CommandLine, True,'', SW_SHOWNORMAL); //Win32s and Win95
        2: RunProcess('cmd.exe /c '+CommandLine, True,'',SW_SHOWNORMAL); // WinNT and Win2000
      end;

      FList := TStringList.Create;
      try

        //load the game txt file into a stringlist.
        Flist.LoadFromFile(MainFrm.Settings.SettingsDir + 'vset.txt');
        For i := 0 to FList.Count-1 do
        begin
          //if the text doesnt start with 'romset' go to the next line
          If not AnsiStartsText('romset' , FList.Strings[i]) then
            continue;

          Name := StrBefore(' ', StrAfter('romset ', FList.Strings[i]));
          TmpROMs.Add( TQPROM.Create() );
          TmpROMs.last.Name := Name;
          TmpROMs.last.CloneName := Name;
          TmpROMs.last.ParentName := StrBetween(FList.Strings[i], '[', ']');
          TmpROMs.last.Path := IncludeTrailingPathDelimiter(ListDirs.Items[Dirs]);
          TmpROMs.last.Emulator := CmbMame.Text;
        end;

      finally
        FreeAndNil(Flist);
      end;

      DeleteFile(MainFrm.Settings.SettingsDir + 'vset.txt');

    end; //end of FOR loop
                          
    //now tmpROMs is full of our search results.  We need to move
    //tmpROMs into the 'proper' ROMlist in the manner the user wants.
    If RadClear.Checked then
    begin
      //Clear the rom list - nice and fast!
      TmpRoms.FileName := MainFrm.GetSelectedTxt;
      TmpRoms.SaveToFile();
    end
    else
    Begin

      //the user wants to retain their existing data.  
      //this is more difficult (and slower)
      With MainFrm do
      begin

        //the first step is to delete entries from the TmpROMs that already exist
        For i := ROMlist.Count-1 downto 0 do
        begin
          FIndex := ROMisIn(ROMlist[i].CloneName, ROMlist[i].Path, TmpROMS);
          If FIndex = -1 then
            ROMlist.Delete(i)
          else
            TmpROMs.Delete(Findex);
        end;

        //now we just need to add the leftover entries from tmpROMs to the ROMlist
        For i := 0 to TmpRoms.Count-1 do
        begin
          ROMlist.Add(TQPROM.Create);
          ROMlist.last.Assign(TmpROMS[i]);
        end;

        //Now save the ROMs objectlist
        ROMlist.SaveToFile();

      end;//end of WITH MainFrm.roms.
    end;

    //now we need to reload the ROMs list.
    MainFrm.ClearROMIcons();
    MainFrm.ROMlist.LoadFromFile(MainFrm.GetSelectedTxt);

  Finally
    FreeAndnil(TmpROMs);
  end;

  //if the CATver option is checked then process the romlist.
  If chkCatver.Checked = true then
    MainFrm.RomList.SetMameCategories(MainFrm.Settings.SettingsDir);

  //now save the users input data.
  Ini := TMemIniFile.Create(MainFrm.Settings.SettingsFile);
  Try

    For i := 0 to ListDirs.Count-1 do
      If DirectoryExists(ListDirs.Items.Strings[i]) then
        DirOutput := DirOutput + ListDirs.Items.Strings[i] + ';';

    Ini.WriteString('MAMEScan', 'Dirs', DirOutput);
    Ini.WriteString('MAMEScan', 'MAME', CmbMAME.Text);
    Ini.WriteBool('MAMEScan', 'CatVer', ChkCatVer.Checked);
    If RadClear.Checked then
      Ini.WriteInteger('MAMEScan', 'ScanType', 1)
    else
      Ini.WriteInteger('MAMEScan', 'ScanType', 2);

    if MainFrm.Settings.AllowWrite then
      Ini.UpdateFile;

  Finally
    FreeAndNil(Ini);
  end;

  //close the form with the modal result OK
  ModalResult := MrOK;
end;

procedure TFrmMAMEMerged.BtnAddClick(Sender: TObject);
begin
  if (jvBrowse.execute) and (jvBrowse.Directory <> '') then
    ListDirs.Items.Add(jvBrowse.Directory);
end;

procedure TFrmMAMEMerged.BtnMinusClick(Sender: TObject);
begin
  ListDirs.DeleteSelected;
end;

function TFrmMAMEMerged.ROMIsIn(Clone, Path: String;
  Searchlist: TQPROMlist): Integer;
var
  I : Integer;
begin
  Result := -1;

  For i := 0 to SearchList.Count-1 do
  begin
    if ( StrCompare(Clone, SearchList[i].CloneName) = 0 ) and ( strCompare(Path, SearchList[i].Path) = 0 ) then
    begin
      Result := i;
      break;
    end;
  end;

end;

end.
