unit fMigrate;

interface

uses
  Windows, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, Contnrs, FindFile, VirtualTrees,
  IniFiles;

type

  TQPArea = (TaROM, TaEmu, TaFav, TaMirror, TaMedia);

  TQPProblem = class
  Private
    FProbDir : String;
    FFixDir : String;
    FArea : TQPArea;
  public
    Property ProbDir : String Read FProbDir Write FProbDir;
    Property FixDir : String Read FFixDir Write FFixDir;
    Property Area : TQPArea Read FArea Write FArea;
    Constructor create(Area : TQPArea; ProbDir, FixDir : String);
  end;

  TQPProbList = class(TObjectList)
  protected
    function GetItem(Index: Integer): TQPProblem;
    procedure SetItem(Index: Integer; AObject: TQPProblem);
    function GetROMCount():Integer;
    function GetEmuCount():Integer;
    function GetFavCount():Integer;
    function GetMediaCount():Integer;
    function GetMirrorCount():Integer;
  public
    property Items[Index: Integer]: TQPProblem read GetItem write SetItem; default;
    Property ROMcount : Integer Read GetROMCount;
    Property EmuCount : Integer Read GetEmuCount;
    Property FavCount : Integer Read GetFavCount;
    Property MirrorCount : Integer Read GetMirrorCount;
    Property MediaCount : Integer Read GetMediaCount;
  end;

  TFrmMigrate = class(TForm)
    PCMigrate: TPageControl;
    TabWelcome: TTabSheet;
    TabResults: TTabSheet;
    lblWelcome: TLabel;
    lblDescription: TLabel;
    BtnScan: TButton;
    BtnCancel: TButton;
    VTMigrate: TVirtualStringTree;
    BtnOK: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure BtnScanClick(Sender: TObject);
    procedure VTMigrateGetText(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
      var CellText: WideString);
    procedure VTMigrateGetImageIndex(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex;
      var Ghosted: Boolean; var ImageIndex: Integer);
    procedure VTMigrateChange(Sender: TBaseVirtualTree;
      Node: PVirtualNode);
    procedure VTMigrateEditing(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex; var Allowed: Boolean);
    procedure VTMigrateNewText(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex; NewText: WideString);
    procedure BtnOKClick(Sender: TObject);
    procedure VTMigrateFreeNode(Sender: TBaseVirtualTree;
      Node: PVirtualNode);
    procedure VTMigrateExit(Sender: TObject);
  private
    { Private declarations }
    ProbList : TQPProbList;
    Procedure PathsToProblems(Paths : TStringList; PType : TQPArea);
    Procedure SetupTree();
  public
    { Public declarations }
  end;

implementation

uses fMain, uRom, JclFileUtils, StrUtils,
  uJVirtualTree, JCLStrings, uRomList, uRomUtils, uQPConst;

{$R *.dfm}

{-----------------------------------------------------------------------------}
{TQPProblem}
{-----------------------------------------------------------------------------}

Constructor TQPProblem.create(Area : TQPArea; ProbDir, FixDir : String);
begin
  FArea := Area;
  FProbDir := ProbDir;
  FFixDir := FixDir;
end;

{-----------------------------------------------------------------------------}
{TQPProbList}
{-----------------------------------------------------------------------------}

function TQPProbList.GetItem(Index: Integer): TQPProblem;
begin
  Result := TQPProblem(inherited Items[Index]);
end;

procedure TQPProbList.SetItem(Index: Integer; AObject: TQPProblem);
begin
  inherited Items[Index] := AObject;
end;

function TQPProbList.GetROMCount():Integer;
var
  i : Integer;
begin
  Result := 0;
  For i := 0 to Count-1 do
    if Items[i].Area = TaROM then
      inc(Result);
end;

function TQPProbList.GetEmuCount():Integer;
var
  i : Integer;
begin
  Result := 0;
  For i := 0 to Count-1 do
    if Items[i].Area = TaEmu then
      inc(Result);
end;

function TQPProbList.GetFavCount():Integer;
var
  i : Integer;
begin
  Result := 0;
  For i := 0 to Count-1 do
    if Items[i].Area = TaFav then
      inc(Result);
end;

function TQPProbList.GetMirrorCount():Integer;
var
  i : Integer;
begin
  Result := 0;
  For i := 0 to Count-1 do
    if Items[i].Area = TaMirror then
      inc(Result);
end;

function TQPProbList.GetMediaCount():Integer;
var
  i : Integer;
begin
  Result := 0;
  For i := 0 to Count-1 do
    if Items[i].Area = TaMedia then
      inc(Result);
end;

{-----------------------------------------------------------------------------}
{TFrmMigrate}
{-----------------------------------------------------------------------------}

Procedure TFrmMigrate.PathsToProblems(Paths : TStringList; PType : TQPArea);
var
  Tmp : TStringList;
  TempDir : String;
  I, e, SamePos : Integer;
begin
  //now we need our second stringlist to hold common paths.
  Tmp := TStringList.Create;
  Try
    //set it up to ignore duplicate entries.
    Tmp.Sorted := True;
    Tmp.Duplicates := dupIgnore;

    //loop through all the paths found in the roms.
    For i := 0 to Paths.Count-1 do
    begin

      //first check that this isnt blank
      If Paths[i] = '' then
        continue;

      //set the tempdir variable equal to the current path.
      TempDir := Paths[i];

      //now loop through all the paths (again)
      For e := 0 to Paths.Count-1 do
      begin
        //we are trying to find the common start between our variable
        //and each entry in the paths stringlist.
        SamePos := PathCommonPrefix(TempDir, Paths[e]);
        //if there is a match then overwrite the temp variable with this match.
        If SamePos > 3 then
          TempDir := Copy(TempDir, 0, SamePos);

      end; //end of for loop

      //finally add the tempdir variable to our tmp stringlist.
      Tmp.Add(TempDir);

    end;

    //now loop through each entry in the tmp stringlist and create object.
    For i := 0 to Tmp.Count-1 do
      ProbList.Add(TQPProblem.create(PType, Tmp[i], Tmp[i]));

  Finally
    FreeAndNil(Tmp);
  end;
end;

{-----------------------------------------------------------------------------}

Procedure TFrmMigrate.SetupTree();
var
  Rnode, FNode, ENode, MirNode, MedNode, CNode : PVirtualNode;
  I : Integer;
begin
  VTMigrate.BeginUpdate;

  VTMigrate.RootNodeCount := 5;
  //the ROM node is the first
  RNode := VTMigrate.GetFirst;
  //second is the Favourites
  FNode := VTMigrate.GetNext(RNode);
  //Emulators is next
  ENode := VTMigrate.GetNext(FNode);
  //Media is Next
  MedNode := VTMigrate.GetNext(Enode);
  //Mirrors are last
  MirNode := VTMigrate.GetNext(MedNode);

  For i := 0 to Problist.Count-1 do
  begin

    case Problist[i].Area of
      TaROM    : CNode := VTMigrate.AddChild(RNode);
      TaFav    : CNode := VTMigrate.AddChild(FNode);
      TaEmu    : CNode := VTMigrate.AddChild(ENode);
      TaMedia  : CNode := VTMigrate.AddChild(MedNode);
      TaMirror : CNode := VTMigrate.AddChild(MirNode);
    else
      Cnode := nil;
    end;

    if CNode <> nil then
      SetObject(VTMigrate, CNode, Problist[i]);
  end;

  VTMigrate.FullExpand();

  VTMigrate.EndUpdate;
end;

{-----------------------------------------------------------------------------}

procedure TFrmMigrate.FormCreate(Sender: TObject);
begin
  //create the objectlist for storing 'problems'
  ProbList := TQPProbList.Create(True);

  //set the node data size fo the virtual tree
  VTMigrate.NodeDataSize := SizeOf(Pointer);

  //set up the page control.
  TabWelcome.TabVisible := False;
  TabResults.TabVisible := False;

  PCMigrate.ActivePage := TabWelcome;
end;

procedure TFrmMigrate.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  FreeAndNil(ProbList);
end;

{-----------------------------------------------------------------------------}

procedure TFrmMigrate.BtnOKClick(Sender: TObject);
var
  TmpROMs : TQPROMList;
  i, e, f, cached : Integer;
  OldCursor : TCursor;
  DecodeStream : TMemoryStream;
  DecodePaths, RomFiles : TStringList;
  Ini : TMemIniFile;
  TmpString : String;
  Find : ThhFindFile;
begin

  //First change the hourglass - this could take a while!
  OldCursor := Screen.Cursor;
  Screen.Cursor := crHourGlass;
  Try

    //the first step is to go through our problems and see if the user has fixed any
    //if they havent fixed the problem, then delete it.
    For i := Problist.Count-1 downto 0 do
      if strCompare(Problist[i].ProbDir, Problist[i].FixDir)=0 then
        Problist.Delete(i); 

    //check to see if there are any problems left, if not then exit.
    //also there is no point continuing if the disk cannot be written to.
    If (Problist.Count = 0) or (MainFrm.Settings.AllowWrite = False) then
      Exit;

    If ProbList.ROMcount > 0 then
    begin
      //first we will fix the ROMs
      RomFiles := TStringList.Create;
      Try

        //set up our find object to get all the ROMs data files.
        uROMUtils.GetAllROMFiles(RomFiles);

        //create our temporary ROMs list, we will need it!
        TmpROMs := TQPROMlist.Create(MainFrm.Settings.AllowWrite);
        Try

          //begin looping through all the rom files.
          For i := 0 to RomFiles.Count-1 do
          begin

            //load this file into the tmproms objectlist.
            TmpROMS.LoadFromFile(RomFiles[i]);
            Cached := -1;
          
            //now loop through each of the roms to get our list of incorrect paths.
            For e := 0 to TmpROMS.Count-1 do
            begin

              if (cached > -1) and (AnsiContainsText(TmpROMS[e].Path, Problist[cached].ProbDir)) then
                TmpROMS[e].Path := AnsiReplaceText(TmpROMS[e].Path, Problist[cached].ProbDir, Problist[cached].FixDir)
              else
              begin
                For f := 0 to ProbList.Count-1 do
                Begin
                  if (Problist[f].Area = TaROM) and (AnsiContainsText(TmpROMS[e].Path, Problist[f].ProbDir)) then
                  begin
                    TmpROMS[e].Path := AnsiReplaceText(TmpROMS[e].Path, Problist[f].ProbDir, Problist[f].FixDir);
                    cached := f;
                    break;
                  end;
                end;
              end;
            end;

            TmpRoms.FileName := RomFiles[i];
            TmpROMS.SaveToFile(); //save the changes to disk.

          end; //end of loop through all files.

        //free the tmproms romlist object.
        Finally
          FreeAndNil(TmpROMs);
        end;

      //free the find object.
      Finally
        FreeAndNil(RomFiles);
      end;

    end; //end of ROMcount check.

    If Problist.EmuCount > 0 then
    begin

      With MainFrm do
      begin
        Cached := -1;
        For i := 0 to EmuList.Count-1 do
        begin
          if (cached > -1) and (AnsiContainsText(EmuList[i].Path, Problist[cached].ProbDir)) then
          begin

            //change the emulators path.
            EmuList[i].Path := AnsiReplaceText(EmuList[i].Path, Problist[cached].ProbDir, Problist[cached].FixDir);

            //check to see if we need to fix the working directory
            if AnsiContainsText(EmuList[i].WorkingDir, Problist[cached].ProbDir) then
              EmuList[i].WorkingDir := AnsiReplaceText(EmuList[i].WorkingDir, Problist[cached].ProbDir, Problist[cached].FixDir);
          end
          else
          For f := 0 to ProbList.Count-1 do
            if (Problist[f].Area = TaEmu) and (AnsiContainsText(EmuList[i].Path, Problist[f].ProbDir)) then
            begin
              //change the emulators path
              EmuList[i].Path := AnsiReplaceText(EmuList[i].Path, Problist[f].ProbDir, Problist[f].FixDir);

              //check to see if the working directory needs changed.
              if AnsiContainsText(EmuList[i].WorkingDir, Problist[f].ProbDir) then
                EmuList[i].WorkingDir := AnsiReplaceText(EmuList[i].WorkingDir, Problist[f].ProbDir, Problist[f].FixDir);

              cached := f;
              break;
            end;
        end;

        EmuList.SaveToFile(); //save the changes

      end;//end of with MainFrms.emulators

    end; // end check that there are emulators to fix.

    If ProbList.FavCount > 0 then
    begin

      With MainFrm do
      begin

        Cached := -1;
        For i := 0 to FavList.Count-1 do
        begin
          if (cached > -1) and (AnsiContainsText(FavList[i].Path, Problist[cached].ProbDir)) then
            FavList[i].Path := AnsiReplaceText(FavList[i].Path, Problist[cached].ProbDir, Problist[cached].FixDir)
          else
          For f := 0 to ProbList.Count-1 do
          begin
            if (Problist[f].Area = TaFav) and (AnsiContainsText(FavList[i].Path, Problist[f].ProbDir)) then
            begin
              FavList[i].Path := AnsiReplaceText(FavList[i].Path, Problist[f].ProbDir, Problist[f].FixDir);
              cached := f;
              break;
            end;
          end;
        end;

        //save the changes to disk.
        FavList.SaveToFile();

      end;//end of WITH MainFrm.favourites

    end;  //end of check that there are favourites to fix.

    //check if there any Media OR mirror paths to fix.
    If (Problist.GetMediaCount > 0) or (ProbList.GetMirrorCount > 0) then
    begin

      Find := ThhFindFile.Create(Self);
      Try
        //create the objects we will need for this
        DecodePaths := TStringList.Create;
        DecodeStream := TMemoryStream.Create;

        //set up the search parameters
        Find.Directory := MainFrm.Settings.ROMsDir;
        Find.Recurse := True;
        Find.Filter := 'folders.ini';

        //start the search
        Find.Execute;

        For i := 0 to Find.TotalFile-1 do
        begin

          Ini := TMemIniFile.Create(Find.Files[i]);
          try

            //load in the media paths for this ini file.
            DecodeStream.Clear;
            Ini.ReadBinaryStream('MediaPaths', 'LstPaths', DecodeStream);
            DecodePaths.LoadFromStream(DecodeStream);

            //sort out all the mirror parts first
            For e := 0 to ProbList.Count-1 do
            begin

              //if this problem is a mirror deal with it!
              If (ProbList[e].Area = TaMirror) and
                 (Ini.ValueExists('Mirror', 'TxtDir')) and
                 (AnsiContainsText(Ini.ReadString('Mirror', 'TxtDir',''), Problist[e].ProbDir)) then
              begin
                TmpString := AnsiReplaceText(Ini.ReadString('Mirror', 'TxtDir',''), Problist[e].ProbDir, Problist[e].FixDir);
                Ini.WriteString('Mirror', 'TxtDir', TmpString);
              end;

              //if this problem is a Media DEAL with it!
              If (ProbList[e].Area = TaMedia) then
              begin

                //see if we need to change the Background path first.
                If (Ini.ValueExists('BkGround', 'TxtBKPath')) and
                   (AnsiContainsText(Ini.ReadString('BkGround', 'TxtBKPath',''), Problist[e].ProbDir)) then
                begin
                  TmpString := AnsiReplaceText(Ini.ReadString('BkGround', 'TxtBKPath',''), Problist[e].ProbDir, Problist[e].FixDir);
                  Ini.WriteString('BkGround', 'TxtBKPath', TmpString);
                end;

                //see if we need to change the Real Icon path.
                If (Ini.ValueExists('RealIcon', 'Directory')) and
                   (AnsiContainsText(Ini.ReadString('RealIcon', 'Directory',''), Problist[e].ProbDir)) then
                begin
                  TmpString := AnsiReplaceText(Ini.ReadString('RealIcon', 'Directory',''), Problist[e].ProbDir, Problist[e].FixDir);
                  Ini.WriteString('RealIcon', 'Directory', TmpString);
                end;

                //now see if we need to change the media paths.
                for f := 0 to DecodePaths.Count-1 do
                  if AnsiContainsText(DecodePaths[f], Problist[e].ProbDir) then
                    DecodePaths[f] := AnsiReplaceText(DecodePaths[f], Problist[e].ProbDir, Problist[e].FixDir);

              end;

            end; //end of sorting out the mirror parts.

            //now put the stream back together.
            DecodeStream.Clear;
            DecodePaths.SaveToStream(DecodeStream);
            DecodeStream.Position := 0;
            Ini.WriteBinaryStream('MediaPaths', 'LstPaths', DecodeStream);

            //remember we need to update the inifile as this is a MEM one.
            Ini.UpdateFile;

          finally
            FreeAndNil(Ini);
          end;

        end; //end of Find file for loop

      Finally
        FreeAndNil(DecodeStream);
        FreeAndNil(DecodePaths);
        FreeAndNil(Find);
      end;

    end; //end of check that there are media/mirrors to fix.

  //return the cursor to normal
  Finally
    Screen.Cursor := OldCursor;
  end;
end;

{-----------------------------------------------------------------------------}

procedure TFrmMigrate.BtnScanClick(Sender: TObject);
var
  Find : ThhFindFile;
  TmpROMs : TQPROMList;
  i, e : Integer;
  OldCursor : TCursor;
  Paths, MirTmp, DecodedPaths : TStringList;
  DecodedStream : TMemoryStream;
  Ini : TMemIniFile;
begin

  //clear the problist objectlist
  ProbList.Clear;

  //First change the hourglass - this could take a while!
  OldCursor := Screen.Cursor;
  Screen.Cursor := crHourGlass;
  Try

    Paths := TStringList.Create;
    Try

      //disable duplicates so we dont fill our stringlist with the same thing!
      Paths.Sorted := True;
      Paths.CaseSensitive := False;
      Paths.Duplicates := dupIgnore;

      //first we will check the ROMs for any invalid directories.
      Find := thhFindFile.Create(Self);
      Try

        //set up our find object to get all the ROMs data files.
        Find.Directory := MainFrm.Settings.ROMsDir;
        Find.Recurse := True;
        Find.Filter := DefaultFileName;
        Find.Attributes := ([]);
        Find.Execute;

        //create our temporary ROMs list, we will need it!
        TmpROMs := TQPROMlist.Create(MainFrm.Settings.AllowWrite);
        Try

          //begin looping through all the rom files.
          For i := 0 to Find.TotalFile-1 do
          begin

            //load this file into the tmproms objectlist.
            TmpROMS.LoadFromFile(Find.Files[i]);
            //now loop through each of the roms to get our list of incorrect paths.
            For e := 0 to TmpROMS.Count-1 do
              Paths.Add(ExtractFilePath(TmpROMS[e].Path));

          end;

          PathsToProblems(Paths, TaROM);
          
        //we have finished with teh tmproms objectlist.
        Finally
          FreeAndNil(TmpROMS);
        end;

        //Reuse the find file object to search for Ini Files.

        //clear the stringlist.
        Paths.Clear;

        Find.Filter := 'folders.ini';
        Find.Execute;

        //we need to create a seperate stringlist to hold mirror results,
        //the media results will be held in our 'paths' stringlist.
        MirTmp := TStringList.Create;
        try

          //this stringlist is needed because i have encoded the media
          //paths in the ini file and now need to decode them.  this sucks.
          DecodedPaths := TStringList.Create;
          try
            DecodedStream := TMemoryStream.Create;

            For i := 0 to Find.TotalFile-1 do
            begin
              Ini := TMemIniFile.Create(Find.Files[i]);
              Try

                //decode the stream containing the media paths.
                Ini.ReadBinaryStream('MediaPaths', 'LstPaths', DecodedStream);
                DecodedPaths.LoadFromStream(DecodedStream);
                Paths.AddStrings(DecodedPaths);
                Paths.Add(Ini.ReadString('BkGround', 'TxtBKPath', ''));
                Paths.Add(Ini.ReadString('RealIcon', 'Directory', ''));
                MirTmp.Add(Ini.ReadString('Mirror', 'TxtDir', ''));

              Finally
                FreeAndNil(Ini);
              end;
            end;

          finally
            FreeAndNil(DecodedStream);
            FreeAndNil(DecodedPaths);
          end;

          //now convert both stringlists into 'problems'
          PathsToProblems(Paths, TaMedia);
          PathsToProblems(MirTmp, TaMirror);
          
        finally
          FreeAndNil(MirTmp);
        end;

      //Free our find object.
      Finally
        FreeAndNil(Find);
      end;

      //END of ROMs checking.

      //clear the Paths stringlist.
      Paths.Clear;

      //now check the emulators for any problems in it.
      For i := 0 to MainFrm.EmuList.Count-1 do
        Paths.Add(ExtractFilePath(MainFrm.EmuList[i].Path));

      PathsToProblems(Paths, TaEmu);

      //clear teh paths stringlist
      Paths.Clear;

      //now check the favourites.
      For i := 0 to MainFrm.FavList.Count-1 do
        Paths.Add(ExtractFilePath(MainFrm.FavList[i].Path));

      PathsToProblems(Paths, taFav);

    //free the paths stringlist
    Finally
      FreeAndNil(Paths);
    end;

    SetupTree();

    PcMigrate.ActivePage := TabResults;
    BtnOK.Visible := True;

  //put the cursor back.
  Finally
    Screen.Cursor := OldCursor;
  end;

end;

{-----------------------------------------------------------------------------}

procedure TFrmMigrate.VTMigrateFreeNode(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
begin
  SetObject(Sender, Node, nil);
end;

{-----------------------------------------------------------------------------}

procedure TFrmMigrate.VTMigrateGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: WideString);
begin
  if TextType = ttNormal then
    case Column of
      0:
        if Node.Parent = Sender.RootNode then
        begin
          // root nodes
          Case Node.Index of
            0 : CellText := QP_ROMS;
            1 : CellText := QP_FAV;
            2 : CellText := QP_EMU;
            3 : CellText := QP_MIGRATE_MISC;
            4 : CellText := QP_MIGRATE_MIRROR;
          end;
            
        end
        else
          CellText := TQPProblem(GetObject(Sender, Node)).ProbDir;

      1: if Node.Parent = Sender.RootNode then
           CellText := ''
         else
           CellText := TQPProblem(GetObject(Sender, Node)).FixDir;
    end;
end;

{-----------------------------------------------------------------------------}

procedure TFrmMigrate.VTMigrateGetImageIndex(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex;
  var Ghosted: Boolean; var ImageIndex: Integer);
begin
  if (Column = 0) and (Node.Parent = Sender.RootNode) then
  begin
    // root nodes
    if Node.Index = 2 then
      ImageIndex := 2
    else
      ImageIndex := 0;
  end
  else
    ImageIndex := -1;
end;

{-----------------------------------------------------------------------------}

procedure TFrmMigrate.VTMigrateChange(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
begin
  with Sender do
  begin
    // Start immediate editing as soon as another node gets focused.
    if Assigned(Node) and (Node.Parent <> RootNode) and not (tsIncrementalSearching in TreeStates) then
    begin
      // Note: the test whether a node can really be edited is done in the OnEditing event.
      EditNode(Node, 1);
    end;
  end;
end;

{-----------------------------------------------------------------------------}

procedure TFrmMigrate.VTMigrateEditing(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; var Allowed: Boolean);
begin
  Allowed := (Column = 1);
end;

{-----------------------------------------------------------------------------}

procedure TFrmMigrate.VTMigrateNewText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; NewText: WideString);
var
  Prob : TQPProblem;
begin
  If (Assigned(Node)) and (NewText <> '') then
  begin
    //assign the selected node to a temporary variable.
    Prob := TQPProblem(GetObject(Sender, Node));

    //if the fix dir ends with a delimiter we need to make sure the
    //users selected directory also does AND vice versa.
    If (IsPathDelimiter(Prob.ProbDir, Length(Prob.ProbDir)) ) then
      Prob.FixDir := IncludeTrailingPathDelimiter(NewText)
    else
      Prob.FixDir := ExcludeTrailingPathDelimiter(NewText);

  end;
end;

{-----------------------------------------------------------------------------}

procedure TFrmMigrate.VTMigrateExit(Sender: TObject);
begin
  if VTMigrate.IsEditing then
    VTMigrate.EndEditNode;
end;

end.
