unit fRunDlg;

interface

uses
  Windows, SysUtils, Classes, Controls, Forms,
  Dialogs, StdCtrls, uRom, uEmu;

type
  TFrmRunRom = class(TForm)
    lblName: TLabel;
    lblPath: TLabel;
    BtnRun: TButton;
    BtnCancel: TButton;
    lblEmu: TLabel;
    lblCommand: TLabel;
    GrpParam: TGroupBox;
    TxtParam: TEdit;
    RadBefore: TRadioButton;
    RadAfter: TRadioButton;
    lblZip: TLabel;
    GrpMisc: TGroupBox;
    ChkWinKey: TCheckBox;
    chkScrSvr: TCheckBox;
    ChkMini: TCheckBox;
    ChkAutoClose: TCheckBox;
    GrpEmu: TGroupBox;
    ChkShortEXE: TCheckBox;
    ChkMame: TCheckBox;
    ChkShortRom: TCheckBox;
    ChkSpeechMarks: TCheckBox;
    ChkMameIncPath: TCheckBox;
    ChkIncFileExt: TCheckBox;
    ChkOnlyROMName: TCheckBox;
    ChkZip: TCheckBox;
    lblZipStatus: TLabel;
    lblZipOutput: TLabel;
    ChkTempDir: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure GenericOnChange(Sender: TObject);
    procedure BtnRunClick(Sender: TObject);
    procedure lblCommandMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    function CommandLine: String;
    function GetExtractionPath(ZipFile : TFileName) : TFileName;
    { Private declarations }
  public
    { Public declarations }
    Rom : TQPRom;
    Emu : TQPEmu;
  end;

implementation

uses fMain, uJUtilities, JCLFileUtils, findFile, Clipbrd, uJFile,
  uJZip, uQPConst;

{$R *.dfm}

Function TFrmRunRom.CommandLine : String;
var
  Cmd, RPath, EPath, Parameters : String;
Begin

   If (not FileExists(ROM.Path)) and
      (not DirectoryExists(ExtractFilePath(ROM.path))) then
   begin
     Result := QP_FILE_NOT_EXIST;
     exit;
   end;

   //set the RPath variable.
   RPath := ROM.Path;

   //reset the zip output label.
   LblZipOutput.Caption := QP_ZIP_NOT_EXTRACTED;

   //if this is a ROM contained in a zip collection we need the file name
   if ROM.ZipName <> '' then
   begin
     If ChkTempDir.Checked then
      RPath := MainFrm.Settings.QPDirectory + 'temp\' + ExtractFileName(ROM.ZipName)
     else
      RPath := ExtractFilePath(ROM.Path) + ExtractFileName(ROM.ZipName);

     lblZipOutput.Caption := RPath;
   end;

   //if the emulator doesnt support zips, get the extraction path.
   If (not ChkZip.Checked) and
      (fileExists(Rpath)) and
      (lowercase(ExtractFileExt(RPath)) = '.zip') then
   begin
     RPath := GetExtractionPath(RPath);
     lblZipOutput.Caption := RPath;
   end;

   //check if the emulator exe is to be launched in short name format
   If ChkShortExe.Checked then
     EPath := ExtractShortPathName(Emu.path)
   else
     EPath := Emu.path;

   //check whether the emulator requires short names 8:3 names
   If ChkShortROM.Checked then
     //RPath := ExtractShortPathName(Rpath);
     Rpath := GetVirtualShortName(Rpath);

   //check whether the emulator requires ROM paths with speech marks
   If ChkSpeechMarks.Checked then
     RPath := '"' + RPath + '"';

   Parameters := TxtParam.Text;

   If (RadBefore.Checked) and (parameters <> '') then
   begin
     Cmd := Parameters;
     If Parameters[Length(Parameters)] = '#' then
       Cmd := Copy(Cmd, 1, Length(Cmd)-1)
     else
       Cmd := Cmd + ' ';
   end;

   // check what format the ROM needs to be in
   If ChkOnlyROMName.Checked then
   begin

     If ROM.CloneName <> '' then
       //ROMs with CloneNames should ignore other settings and use the clone name
       Cmd := Cmd + ROM.CloneName
     else
     begin
       //its not MAME so carry on.
       if ChkIncFileExt.Checked then
         Cmd := Cmd + ExtractFileName(Rpath)
       else
         Cmd := Cmd + ChangeFileExt(ExtractFileName(RPath), '');
     end;

   end
   else //else the full path to the ROM is needed.
     Cmd := Cmd + Rpath;

   If ChkMAMe.Checked and ChkMameIncPath.Checked then
     Cmd := Cmd + ' -rompath ' + ExtractFilePath(RPath) + ' ';

   //add in the file parameters if they are not blank
   If (RadAfter.Checked) And (Parameters <> '') then
   begin
     If Parameters[1] = '#' then
       Cmd := Cmd + Copy(Parameters, 2, Length(Parameters))
     else
       Cmd := Cmd + ' ' + Parameters;
   end;

   result := Epath+' '+Cmd;
End;

function TFrmRunRom.GetExtractionPath(ZipFile : TFileName) : TFileName;
var
  I : Integer;
  UseDir : String;
  ZList : TStrings;
begin

  //this function will get the proposed file path after a zip has been extracted.
  Result := '';

  with MainFrm do
  begin

    If ChkTempDir.Checked then
      UseDir := Settings.QPDirectory + 'temp\'
    else
      UseDir := ExtractFilePath(ZipFile);

    zList := TStringList.Create;
    try
      GetZipContents(zList, ZipFile);

      If zList.Count = 1 then
      Result := UseDir + ExtractFileName(zList[0]);

      //if more than one file is extracted we have a problem!
      //Just try to GUESS which file is the rom... pants...
      If zList.Count > 1 then
        For i := 0 to zList.Count-1 do
          If (ExtractFileExt(zList[i]) <> '.txt') or
            (ExtractFileExt(zList[i]) <> '.nfo') or
            (ExtractFileExt(zList[i]) <> '.diz') then
          Begin
            Result := UseDir + ExtractFileName(zList[i]);
            Break;
          End;
    finally
      FreeAndNil(zList);
    end;

  end;//end of with MainFrm.

end;

procedure TFrmRunRom.FormShow(Sender: TObject);
var
  I : Integer;
begin
  With MainFrm, MainFrm.Settings do
  Begin

    lblName.Caption := Rom.name;
    LblPath.Caption := Rom.path;
    LblEmu.Caption := Emu.name;

    If FileExists(Rom.path) then
    begin
      If lowercase(ExtractFileExt(Rom.path)) = '.zip' then
        if VerifyZip(Rom.path) then
          LblZip.Caption := QP_YES
        Else
          lblZip.Caption := QP_NO
      else
      begin
        lblZip.Caption := QP_ZIP_NON_ZIP_FILE;
        lblZipOutput.Caption := QP_ZIP_NON_ZIP_FILE;
      end;
    end
    Else if DirectoryExists(ROM.Path) then
    begin
      lblZip.Caption := QP_FILE_IS_DIRECTORY;
      lblZipOutput.Caption := QP_RUNDLG_MAME_ROM;
    end
    else
      LblZip.Caption := QP_FILE_NOT_EXIST;

    ChkMini.Checked := MiniOnRun;
    ChkTempDir.Checked := UseTempDir;

    ChkSpeechMarks.Checked := Emu.UseSpeechMarks;
    ChkMame.Checked := Emu.mame;
    ChkMameIncPath.Checked := Emu.MameIncPath;
    ChkOnlyROMName.Checked := Emu.OnlyROMName;
    ChkIncFileExt.Checked := Emu.IncROMExt;
    ChkZip.Checked := Emu.zip;
    ChkShortEXE.Checked := Emu.ShortExe;
    ChkShortRom.Checked := Emu.ShortRom;
    ChkScrSvr.Checked := Emu.DisScrSvr;
    ChkWinKey.Checked := Emu.DisWinKey;

    If (Rom.Overwrite) AND (Rom.Parameters <> '') then
      TxtParam.Text := Rom.Parameters
    Else
      TxtParam.Text := Emu.parameters + Rom.Parameters;

    If Emu.ParamBefore then
      RadBefore.Checked := True
    Else
      RadAfter.Checked := True;

    LblCommand.Caption := CommandLine();

    //set up all the events - we do this after setting up to avoid executing
    //the code everytime we change at startup!
    for i := 0 to GrpEmu.ControlCount-1 do
      TCheckBox(GrpEmu.Controls[i]).OnClick := GenericOnChange;

    TxtParam.OnChange := GenericOnChange;
    RadAfter.OnClick := GenericOnChange;
    RadBefore.OnClick := GenericOnChange;
    ChkTempDir.OnClick := GenericOnChange;

  end;//end of WITH block

end;

Procedure TFrmRunRom.GenericOnChange(Sender : TObject);
Begin
  lblCommand.Caption := CommandLine;
End;

procedure TFrmRunRom.BtnRunClick(Sender: TObject);
var
  OldMiniSetting : Boolean;
  OldTempSetting : Boolean;
  CopyEmu : TQPEmu;
begin

  //hide the run dialog
  If ChkMini.Checked then
    Self.Visible := False;

  //change specific settings.
  OldMiniSetting := MainFrm.Settings.MiniOnRun;
  OldTempSetting := MainFrm.Settings.UseTempDir;
  MainFrm.Settings.UseTempDir := ChkTempDir.Checked;
  MainFrm.Settings.MiniOnRun := ChkMini.Checked;

  CopyEmu := TQPEmu.Create;
  try
    
    CopyEmu.Assign(Emu);

    //now we have a copy of the emulator object to work with
    CopyEmu.Zip := ChkZip.Checked;
    CopyEmu.ShortExe := ChkShortExe.Checked;
    CopyEmu.ShortRom := ChkShortROM.Checked;
    CopyEmu.UseSpeechMarks := ChkSpeechMarks.Checked;
    CopyEmu.Mame := ChkMame.Checked;
    CopyEmu.MameIncPath := ChkMameIncPath.Checked;
    CopyEmu.OnlyROMName := ChkOnlyROMName.Checked;
    CopyEmu.IncROMExt := ChkIncFileExt.Checked;
    CopyEmu.DisScrSvr := ChkScrSvr.Checked;
    CopyEmu.DisWinKey := ChkWinKey.Checked;
    CopyEmu.Parameters := TxtParam.Text;
    If RadBefore.Checked then
      CopyEmu.ParamBefore := True
    else
      CopyEmu.ParamBefore := False;

    //now we have the custom settings in place, run the rom.
    MainFrm.RunROM(Rom, CopyEmu);

  finally
    MainFrm.Settings.MiniOnRun := OldMiniSetting;
    MainFrm.Settings.UseTempDir := OldTempSetting;
    FreeAndNil(CopyEmu);
  end;

  //bring back the Run Dialog
  If ChkMini.Checked and Self.Visible = false then
    Self.Visible := True;

  If ChkAutoClose.Checked then
    //if autoclose is enabled close the form.
    ModalResult := MrOK
  else
    //we need to bring this form to the front.
    Self.BringToFront;

end;

procedure TFrmRunRom.lblCommandMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Clip : TClipBoard;
begin
  If Button = mbRight then
  begin
    Clip := ClipBoard;
    Clip.AsText := lblCommand.Caption;
  end;
end;

end.
