unit fScan;

interface

uses
  Windows, SysUtils, Classes, Controls, Forms, Dialogs,
  FindFile, StdCtrls, Buttons, ComCtrls, JvBaseDlg,
  JvBrowseFolder, JvComponent;

type
  TScanForm = class(TForm)
    FindFile: ThhFindFile;
    GrpOptions: TGroupBox;
    CmbEmu: TComboBox;
    GrpFolder: TGroupBox;
    ChkSubDir: TCheckBox;
    BtnScan: TButton;
    BtnCancel: TButton;
    jvBrowse: TJvBrowseForFolderDialog;
    BtnMinus: TSpeedButton;
    BtnAdd: TSpeedButton;
    ListDirs: TListBox;
    lblEmu: TLabel;
    lblSystem: TLabel;
    CmbSystem: TComboBox;
    ChkCatver: TCheckBox;
    ChkDup: TCheckBox;
    GrpLanguage: TGroupBox;
    RadEnglish: TRadioButton;
    RadNoLang: TRadioButton;
    RadAutoLang: TRadioButton;
    FilterGroup: TGroupBox;
    BtnAddFilter: TSpeedButton;
    BtnFilterMinus: TSpeedButton;
    FilterList: TListBox;
    GrpRatings: TGroupBox;
    RadNoRating: TRadioButton;
    RadSetRating: TRadioButton;
    CmbRating: TComboBox;
    ChkInsideZips: TCheckBox;
    procedure BtnScanClick(Sender: TObject);
    procedure BtnAddFilterClick(Sender: TObject);
    procedure BtnFilterMinusClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure CmbSystemChange(Sender: TObject);
    procedure RadSetRatingClick(Sender: TObject);
    procedure RadNoRatingClick(Sender: TObject);
    procedure BtnAddClick(Sender: TObject);
    procedure BtnMinusClick(Sender: TObject);

  private
    FPreviousScanDir : String;
    FCurrentDir : string;
    procedure init();
    
  public
    { Public declarations }
  end;

implementation

uses fMain, uJUtilities, uEmuList, uRomList, inifiles, JCLStrings, uQPConst;

{$R *.DFM}

{-----------------------------------------------------------------------------}

procedure TScanForm.BtnAddFilterClick(Sender: TObject);
begin
  filterlist.items.Add(inputBox(QP_FOLOPTFRM_FILE_EXT,QP_FOLOPTFRM_FILE_EXT,''));
end;

{-----------------------------------------------------------------------------}

procedure TScanForm.BtnFilterMinusClick(Sender: TObject);
begin
  filterlist.DeleteSelected;
end;

{-----------------------------------------------------------------------------}

procedure TScanForm.BtnScanClick(Sender: TObject);
var
  i: integer;
  filter, Rating : string;
  Lang : TQPLanguageMode;
  Ini : TMemIniFile;
  OldStyle : Tcursor;
begin
  OldStyle := screen.Cursor;
  Screen.Cursor := crHourGlass;
  try

//This Section of code initialises + Executes FindFile.
    if (ListDirs.Count > 0) then
    begin

        //set up the search options
        if FilterList.Items.Count > 0 then
        Begin
          for i := 0 to (filterlist.Items.Count - 1) do
            filter := filter + ';' + filterlist.items[i];
        end
        else
        Begin
          MessageDlg(QP_SCAN_NO_FILTER, mtWarning, [mbOK], 0);
          ModalResult := MrNone;
          Exit;
        end;

        if RadSetRating.Checked then
          Rating := CmbRating.Text
        else
          Rating := '';

        if RadAutoLang.Checked then
          Lang := lmGuess
        else if RadEnglish.Checked then
          Lang := lmEnglish
        else
          Lang := lmNone;

        //now save whatever is left over
        MainFrm.RomList.AddFromDirs(ListDirs.Items, Filter, ChkSubDir.Checked,
        ChkInsideZips.Checked, CmbEmu.Text, Rating, Lang, ChkDup.Checked);

        If chkCatver.Checked = true then
          MainFrm.RomList.SetMameCategories(MainFrm.Settings.SettingsDir);

    end
    Else
    begin
      Screen.Cursor := OldStyle;
      MessageDlg(QP_ENTER_ALL_DATA, mtError, [mbOK], 0);
      modalresult := mrNone;
    end;

    Ini := TMemIniFile.Create(MainFrm.Settings.SettingsFile);
    try

      Ini.WriteString('Scan', 'LastDir', FPreviousScanDir);

      case ChkSubDir.checked of
        false : Ini.WriteInteger('Scan','SubChk', 0);
        true : Ini.WriteInteger('Scan','SubChk', 1);
      end;

      case ChkDup.checked of
        false : Ini.WriteInteger('Scan','DupChk', 0);
        true : Ini.WriteInteger('Scan','DupChk', 1);
      end;

      case ChkCatVer.checked of
        false : Ini.WriteInteger('Scan','ChkCatVer', 0);
        true : Ini.WriteInteger('Scan','ChkCatVer', 1);
      end;

      Ini.WriteString('Scan','LastEmu', CmbEmu.text);

      if MainFrm.Settings.AllowWrite then
        Ini.UpdateFile;

    finally
      FreeAndNil(Ini);
    end;

  finally
    Screen.Cursor := OldStyle;
  End;
end;

{-----------------------------------------------------------------------------}
{                              Private Routines                               }
{-----------------------------------------------------------------------------}

procedure TScanForm.init();
var
  Ini : TMemIniFile;
begin

  CmbSystem.Items.LoadFromFile(MainFrm.Settings.SettingsDir+'systems.dat');

  CmbSystem.Items.Insert(0, 'All');
  CmbSystem.ItemIndex := 0;

  //Set up the emulators Combo Box
  CmbEmu.Items.Clear;
  MainFrm.EmuList.EmusToStrings(CmbEmu.Items, cfAllEmus);

  Ini := TMemIniFile.Create(MainFrm.Settings.SettingsFile);
  Try

    FPreviousScanDir := Ini.ReadString('Scan', 'LastDir', '');

    if Ini.ValueExists('Scan','SubChk') then
      case (Ini.ReadInteger('Scan','SubChk',0)) of
        0: ChkSubDir.checked := false;
        1: ChkSubDir.Checked := true;
      end;

    //Check whether duplicate checking is turned on
    if Ini.ValueExists('Scan','DupChk') then
      case (Ini.readInteger('Scan','DupChk',0)) of
        0: chkdup.checked := false;
        1: chkdup.checked := true;
      end;

    //Check whether use Catver.ini is turned on
    if Ini.ValueExists('Scan','ChkCatver') then
      Case (Ini.ReadInteger('Scan','ChkCatver',0)) of
        0: chkCatVer.checked := false;
        1: ChkCatVer.Checked := true;
      end;

    //Load in the last selected emulator
    if Ini.ValueExists('Scan','LastEmu') then
        CmbEmu.ItemIndex := CmbEmu.Items.IndexOf(Ini.ReadString('Scan','LastEmu',''));

  Finally
    FreeAndNil(Ini);
  end;

  FCurrentDir := MainFrm.GetSelectedTxt;

end;

procedure TScanForm.FormCreate(Sender: TObject);
begin
  init();
end;

procedure TScanForm.CmbSystemChange(Sender: TObject);
begin

  With MainFrm.EmuList do
  Begin
    CmbEmu.Items.Clear;
                  
    If CmbSystem.Text = QP_ALL then
      EmusToStrings(CmbEmu.Items, cfAllEmus)
    else
      EmusToStrings(CmbEmu.Items, cfEmuBySys, CmbSystem.Text);


  end;//end of WITH block
end;

procedure TScanForm.RadSetRatingClick(Sender: TObject);
begin
  CmbRating.Enabled := RadSetRating.Checked;
end;

procedure TScanForm.RadNoRatingClick(Sender: TObject);
begin
  CmbRating.Enabled := RadSetRating.Checked;
end;

procedure TScanForm.BtnAddClick(Sender: TObject);
begin

  //All this code is to get the initally selected directory.
  if ListDirs.ItemIndex <> -1 then
    jvBrowse.Directory := ListDirs.Items[ListDirs.itemIndex]
  else
    jvBrowse.Directory := FPreviousScanDir;

  //now actually execute the dialog and add the result to the list.
  if (jvBrowse.execute) and (DirectoryExists(jvBrowse.Directory)) then
  begin

    //add the directory to the list
    ListDirs.Items.Add(jvBrowse.Directory);

    //change the last previous scan directory to this one
    FPreviousScanDir := jvBrowse.Directory;
    
  end;
end;

procedure TScanForm.BtnMinusClick(Sender: TObject);
begin
  ListDirs.DeleteSelected;
end;

end.
