unit uEmu;

interface

uses Windows, Forms, SysUtils, Dialogs, Controls, Classes, uRom;

type

  TQPSystem = class
  Protected
    Fname : string;
    FTotalEmus : Integer;
  Public
    Property Name : String Read FName Write Fname;
    Property TotalEmus : Integer read FTotalEmus Write FTotalEmus;
  end;

  TQPEmu = Class
  Private
    _name,
    _Version,
    _path,
    _cfg,
    _system,
    _WorkingDir,
    _HomePage,
    _parameters: string;
    _CmdLine,
    _UseLaunchApp,
    _ShellEx,
    _ParamBefore,
    _zip,
    _mame,
    _MameUsePath,
    _ShortExe,
    _ShortRom,
    _OnlyROMName,
    _IncludeROMExt,
    _UseSpeechMarks,
    _DisScrSvr,
    _DisWinKey,
    _Verify : Boolean;
    _IconIndex : Integer;  //only really used on the emulator management screen.
  public
    Property Name : String Read _Name Write _Name;
    Property Version : String Read _Version Write _Version;
    Property Path : String Read _path Write _path;
    Property Cfg : String Read _cfg Write _cfg;
    Property System : String Read _system Write _system;
    Property WorkingDir : String Read _WorkingDir Write _WorkingDir;
    Property HomePage : String Read _HomePage Write _HomePage;
    Property Parameters : String Read _parameters Write _parameters;
    Property CmdLine : Boolean Read _CmdLine Write _CmdLine;
    Property ShellEx : Boolean Read _ShellEx Write _ShellEx;
    Property UseLaunchApp : Boolean Read _UseLaunchApp write _UseLaunchApp;
    Property ParamBefore : Boolean Read _ParamBefore Write _ParamBefore;
    Property Zip : Boolean Read _zip Write _zip;
    Property Mame : Boolean Read _mame Write _mame;
    Property MameIncPath : Boolean Read _MameUsePath Write _MameUsePath;
    Property OnlyROMName : Boolean Read _OnlyROMName Write _OnlyROMName;
    Property IncROMExt : Boolean Read _IncludeROMExt Write _IncludeROMExt;
    Property ShortExe : Boolean Read _ShortExe Write _ShortExe;
    Property ShortRom : Boolean Read _ShortRom Write _ShortRom;
    Property UseSpeechMarks : Boolean Read _UseSpeechMarks Write _UseSpeechMarks;
    Property DisScrSvr : Boolean Read _DisScrSvr Write _DisScrSvr;
    Property DisWinKey : Boolean Read _DisWinKey Write _DisWinKey;
    Property Verify : Boolean Read _Verify Write _Verify;
    Property IconIndex : Integer Read _IconIndex Write _IconIndex;

    constructor Create;
    procedure Assign(Source : TQPEmu);
    Function BuildCmdLine(RPath : String; RomObj : TQPRom; OnlyParams : Boolean = False): String;
    procedure MameConfigCheck();
    procedure MameCreateDataFile(DataDir : String);
    procedure OpenHomePage();
    procedure RunROM(RomObj : TQPRom);
  end;

implementation

{$I Compilers.inc}

uses uQPConst, ShellAPI, JCLShell, uJUtilities, uJZip;

procedure TQPEmu.Assign(Source: TQPEmu);
begin
  _name := Source.Name;
  _Version := Source.Version;
  _path := Source.Path;
  _cfg := Source.Cfg;
  _system := Source.System;
  _WorkingDir := Source.WorkingDir;
  _HomePage := Source.HomePage;
  _Parameters := Source.Parameters;
  _CmdLine := Source.CmdLine;
  _ShellEx := Source.ShellEx;
  _UseLaunchApp := Source.UseLaunchApp;
  _ParamBefore := Source.ParamBefore;
  _zip := Source.Zip;
  _mame := Source.Mame;
  _MameUsePath := Source.MameIncPath;
  _OnlyROMName := Source.OnlyROMName;
  _IncludeROMExt := Source.IncROMExt;
  _ShortExe := Source.ShortExe;
  _ShortRom := Source.ShortRom;
  _UseSpeechMarks := Source.UseSpeechMarks;
  _DisScrSvr := Source.DisScrSvr;
  _DisWinKey := Source.DisWinKey;
  _Verify := Source.Verify;
  _IconIndex := Source.IconIndex;
end;

Function TQPEmu.BuildCmdLine(RPath : String; RomObj : TQPRom; OnlyParams : Boolean = False) : String;
var
  Cmd, EPath, Parameters : String;
Begin

   //check if the emulator exe is to be launched in short name format
   If _ShortExe then
     EPath := ExtractShortPathName(_path)
   else
     EPath := _path;

   //check whether the emulator requires short names 8:3 names
   If _ShortRom then
     RPath := ExtractShortPathName(Rpath);

   //check whether the emulator requires ROM paths with speech marks
   If _UseSpeechMarks then
     RPath := '"' + RPath + '"';

   //build the parameter list
   IF (RomObj.Overwrite) AND (RomObj.Parameters <> '') then
     Parameters := RomObj.Parameters
   Else
     Parameters := Self.parameters + RomObj.Parameters;

   If (_ParamBefore) and (parameters <> '') then
   begin
     Cmd := Parameters;
     If Parameters[Length(Parameters)] = '#' then
       Cmd := Copy(Cmd, 1, Length(Cmd)-1)
     else
       Cmd := Cmd + ' ';
   end;

   // check what format the ROM needs to be in
   If _OnlyROMName then
   begin

     If RomObj.CloneName <> '' then
       //ROMs with CloneNames should ignore other settings and use the clone name
       Cmd := Cmd + RomObj.CloneName
     else
     begin
       //its not MAME so carry on.
       if _IncludeROMExt then
         Cmd := Cmd + ExtractFileName(Rpath)
       else
         Cmd := Cmd + ChangeFileExt(ExtractFileName(RPath), '');
     end;

   end
   else //else the full path to the ROM is needed.
     Cmd := Cmd + Rpath;

   If _Mame and _MameUsePath then
     Cmd := Cmd + ' -rompath ' + ExtractFilePath(RPath) + ' ';

   //add in the file parameters if they are not blank
   If (_ParamBefore = False) And (Parameters <> '') then
   begin
     If Parameters[1] = '#' then
       Cmd := Cmd + Copy(Parameters, 2, Length(Parameters))
     else
       Cmd := Cmd + ' ' + Parameters;
   end;

   if OnlyParams then
    Result := Cmd
   else
    result := Epath+' '+Cmd;

   {$IFDEF DEBUG}
   AddToLog('Commandline - ' + Result);
   {$ENDIF}
   
End;


constructor TQPEmu.Create;
Begin
  _name := '';
  _Version := '';
  _path := '';
  _cfg := '';
  _system := '';
  _WorkingDir := '';
  _HomePage := '';
  _parameters := '';
  _CmdLine := true;
  _shellEx := False;
  _ParamBefore := False;
  _zip := False;
  _mame := False;
  _MameUsePath := False;
  _OnlyROMName := False;
  _IncludeROMExt := True;
  _ShortExe := False;
  _ShortRom := False;
  _UseSpeechMarks := False;
  _DisScrSvr := true;
  _DisWinKey := true;
  _Verify := false;
  _IconIndex := -1;
End;

{-----------------------------------------------------------------------------}

Procedure TQPEmu.MameConfigCheck();
Var
  CommandLine : String;
Begin
  If Self.Mame then
    If not FileExists(ExtractFilePath(_Path)+'mame.ini') and
       not FileExists(ExtractFilePath(_Path)+'mame.cfg') then
      if MessageDlg(QP_MAME_INI_NOT_FOUND,
            mtConfirmation, [mbYes, mbNo], 0) = mrYes then
      begin
        CommandLine := ExtractShortPathName(_Path) + ' -createconfig';
        case Win32Platform of
        0,1: RunProcess('command.com /c '+CommandLine, True, ExtractFilePath(_Path), SW_SHOWMINIMIZED); //Win32s and Win95
        2: RunProcess('cmd.exe /c '+CommandLine, True, ExtractFilePath(_Path),SW_SHOWMINIMIZED); // WinNT and Win2000
        end;

      end;

End;

{-----------------------------------------------------------------------------}

Procedure TQPEmu.MameCreateDataFile(DataDir : String);
var
  GameDoc,
  NameDoc,
  commandline,
  RealName, IniName, Err : String;
  I, e, gpos : Integer;
  InputName, InputGame, OutputFile : TStringList;
  temp : String[8];
Begin
  // this is a complicated function - basically it will create a MAME.ini file
  // it needs the name of the ini file - based on the name of the emulator
  // and the path to MAME

  IniName := DataDir + _name + '.ini';

  If not FileExists(IniName) then
  begin

    // the process of creating the QP MAME.ini file is 2 fold
    // firstly we need to create the 2 temp files which hold ROM information
    // secondly we convert these 2 files into our final product.

    // get our temporary names and delete the files if they already exist
    NameDoc := ExtractShortPathName(DataDir) + 'name.tmp';
    GameDoc := ExtractShortPathName(DataDir) + 'game.tmp';

    If FileExists(NameDoc) then
      DeleteFile(NameDoc);
    If FileExists(GameDoc) then
      DeleteFile(GameDoc);

    //generate the listfull txt document
    commandline := extractshortpathname(Self._path) +' -listfull -sortname > ' + namedoc;
    case Win32Platform of
      0,1: Err := RunProcess('command.com /c '+CommandLine, True,'', SW_SHOWMINNOACTIVE	); //Win32s and Win95
      2: Err := RunProcess('cmd.exe /c '+CommandLine, True,'',SW_SHOWMINNOACTIVE	); // WinNT and Win2000
    end;

    if Err <> '' then
      raise EJException.Create(QP_MAME_GENERATE_DATA_ERR + ' - ' + Err);

    //generate the listgames txt document
    commandline := extractshortpathname(Self._path) +' -listgames -sortname > ' + gamedoc;
    case Win32Platform of
      0,1: Err := RunProcess('command.com /c '+CommandLine, True,'', SW_SHOWMINNOACTIVE	); //Win32s and Win95
      2: Err := RunProcess('cmd.exe /c '+CommandLine, True,'',SW_SHOWMINNOACTIVE	); // WinNT and Win2000
    end;

    if Err <> '' then
      raise EJException.Create(QP_MAME_GENERATE_DATA_ERR + ' - ' + Err);

    //now we have 2 tmp files containing MAME rom information
    //create our Ini object to store our new data

    InputName := TStringList.Create;
    try
      InputGame := TStringList.Create;
      OutputFile := TStringList.Create;
      //load the first MAME tmp file into our stringlist
      InputName.LoadFromFile(NameDoc);
      InputGame.LoadFromFile(GameDoc);

      //add in some custom QP information
      OutputFile.Add('[Quickplay Internal Data]');
      OutputFile.Add('Version=3.6 Generated File');
      OutputFile.Add('MamePath=' + _path);
      OutputFile.Add('');

      //skip the first line of the file
      For i := 1 to InputName.Count-1 do
      Begin
        temp := InputName.Strings[i];
        RealName := Copy(InputName.Strings[i], 12, Length(InputName.Strings[i])-12 );
        OutputFile.Add('['+ trim(temp) +']');
        OutputFile.Add('name=' + RealName);

        gpos := -1;
        //now we need to find the year of the game?
        for e := 0 to InputGame.Count-1 do
        begin
          if Pos(RealName, InputGame.Strings[e]) <> 0 then
          begin
            gpos := e;
            break;
          end;
        end;

        if gpos <> -1 then
        begin
          OutputFile.Add('year='+Copy(InputGame.Strings[gpos],0,4));
          InputGame.Delete(gpos);
        end
        else
          OutputFile.Add('year=unknown');

        //add in the spacer
        OutputFile.Add('');

      end;

      //save our output file to disk
      OutputFile.SaveToFile(IniName);

    finally
      FreeAndNil(InputName);
      FreeAndNil(InputGame);
      FreeAndNil(OutputFile);
    end;

    // time to clean up any files we have left around!
    If FileExists(GameDoc) then
      DeleteFile(GameDoc);
    If FileExists(NameDoc) then
      DeleteFile(NameDoc);

    // and give one final check that the ini exists before ending the function
    If not FileExists(IniName) then
      raise EJException.Create(QP_MAME_GENERATE_DATA_ERR);

  end;

End;

procedure TQPEmu.OpenHomePage();
begin
  // this procedure loads the home page of the emulator

  //if the homepage is not a blank entry load it
  If _HomePage <> '' then
    ShellExecute(Application.Handle, 'open', PChar(_HomePage), '', '', sw_Show);
end;

{-----------------------------------------------------------------------------}

procedure TQPEmu.RunROM(RomObj : TQPRom);
var
  i : Integer;
  RPath, HelpCmd : string;
  ZipList : TStringList;
begin

  {$IFDEF DEBUG}
  AddToLog('Start of GAME LAUNCHING CODE----------');
  {$ENDIF}
  
  ZipList := TStringList.Create;
  Try
    RPath := RomObj.Path;
    //First check that the emulator and rom arent blank AND they exist
    if (FileExists(RPath) or DirectoryExists(ExtractFilePath(RPath)) )
    and FileExists(_Path) then
    Begin

      // check that the found emulator supports cmd lines, if it doesnt
      // tell the user and exit
      If (not _CmdLine) and (not _ShellEx) then
      begin
        {$IFDEF DEBUG}
        AddToLog('LaunchROM - Emulator doesnt support cmd lines, ' + _Name);
        {$ENDIF}
        raise EJException.Create(QP_EMU_DOESNT_SUPPORT_CMD);
      end;

      //verify the integrity of the ZIP file
      If (_Verify) and ((lowercase(extractFileExt(RomObj.path)) = '.zip')) then
        If VerifyZip(RomObj.path) = false then
          raise EJException.Create(QP_ZIP_CORRUPT);

      //disable the winkeys if required
      If _DisWinKey then
        WinKeyOnOff(false);

      //disable the screensaver if required
      If _DisScrSvr  then
        ScreenSaverOnOff(False);

      If RomObj.ZipName = '' then
      begin
        RPath := RomObj.path;
        {$IFDEF DEBUG}
        AddToLog('ROM NOT in Zip - ' + RomObj.path);
        {$ENDIF}
      end
      else
      begin

        //This means the rom is contained inside a zip collection and
        //needs to be extracted before it can be run.
        try
          UnzipSingleFile(RomObj.Path, ExtractFilePath(RomObj.Path), RomObj.ZipName);
        except
          on e: Exception do Raise EJException.Create(E.Message);
        end;

        If FileExists(ExtractFilePath(RomObj.Path) + ExtractFileName(RomObj.ZipName)) then
          RPath := ExtractFilePath(RomObj.Path) + ExtractFileName(RomObj.ZipName);

        {$IFDEF DEBUG}
        AddToLog('ROM was in Zip File (' + RomObj.Path + ') extracted to ' + RPath);
        {$ENDIF}
      end;

      //if the emulator does not support zips then it needs to be extracted
      If (_zip = False) AND (lowercase(extractFileExt(RPath)) = '.zip') then
      begin
        RPath := UnzipRom(ziplist, Rpath);
        {$IFDEF DEBUG}
        AddToLog('Emu Doesnt Support Zips, ROM extracted to : ' + RPath);
        {$ENDIF}
      end;

      If RPath = '' then
        raise EJException.Create(QP_GET_ROM_PATH_FAILED);
      
      If _Mame then
        Self.MameConfigCheck();

      {$IFDEF DEBUG}
      AddToLog('-------Running Process/ShellEx-ing---------');
      {$ENDIF}
      
      //launch the emulator and rom
      If _ShellEx then
        ShellExecAndWait(RPath)
      else if Self._UseLaunchApp then
      begin
        //use the launcher help application.
        HelpCmd := 'qplauncher.exe' + ' ' +
          StringReplace(_path, ' ', '', [rfReplaceAll]) + ' ' +
          StringReplace(Self.BuildCmdLine(RPath, RomObj, True), ' ', '', [rfReplaceAll]) + ' ' + 
          StringReplace(Self._WorkingDir, ' ', '', [rfReplaceAll]);
        RunProcess(HelpCmd);
        raise EJNotice.Create('TERMINATE');
      end
      else
        RunProcess(Self.BuildCmdLine(RPath, RomObj), True, Self._WorkingDir);
    
      //increment the roms numplayed
      RomObj.NumPlay := RomObj.NumPlay + 1;

      For i := 0 to ZipList.Count-1 do
        If FileExists(ZipList[i]) and not (Self._UseLaunchApp) then
          DeleteFile(ZipList[i]);

      //if the file was part of a zip collection we need to delete it
      If (RomObj.ZipName <> '') and not (Self._UseLaunchApp) and 
         (fileExists(ExtractFilePath(RomObj.Path) + ExtractFileName(RomObj.ZipName))) then
          deleteFile(ExtractFilePath(RomObj.Path) + ExtractFileName(RomObj.ZipName));

      //re-enable the screensaver if required
      If _DisScrSvr then
        ScreenSaverOnOff(True);

      //re-enable the win keys if required
      If _DisWinKey then
        WinKeyOnOff(true);

    end //end of IF file exists code
    else // give an error message
      raise EJException.create(QP_GENERIC_RUN_FAILED);
    
  finally
    FreeAndNil(ZipList);
  end;

  {$IFDEF DEBUG}
  AddToLog('End of GAME LAUNCHING CODE----------');
  {$ENDIF}

end;

{-----------------------------------------------------------------------------}

end.
 
