unit uJFile;

interface

uses Windows, Classes, SysUtils, Graphics;

  Function DirCheck(Dir : String; DirType : String) : Boolean;
  procedure DirCopy(FromDir, ToDir : string);
  function ExeTypeToString(const fName: string): String;
  procedure FileCopy(Source, Destination : String; DelSource : Boolean = false);
  Function FileIsImage(InFile : TFileName) : Boolean;
  function GetEXEIcon(var myIcon : TIcon; filename : String) : Boolean;
  function GetVirtualShortName(FileName : TFileName) : String;
  function HasSubDirs(const Folder: string): Boolean;
  function IsDriveReadOnly(Path : String) : Boolean;

implementation

{$I Compilers.inc}

uses jvSHFileOperation, JCLShell, ShellAPI, JCLStrings, uJUtilities, Dialogs, uQPConst;

{-----------------------------------------------------------------------------}

Function DirCheck(Dir : String; DirType : String) : Boolean;
begin

  Result := true;
  If Not DirectoryExists(Dir) then
  begin

    If DirType = QP_INPUT then
    begin
      MessageDlg(QP_DIRCHECK_1 + DirType + QP_DIRECTORY_NOT_EXIST, mtError, [mbOK], 0);
      Result := False;
    end

    else

    if DirType = QP_OUTPUT then

      If NOT (CreateDir(Dir)) Then
      begin
        MessageDlg(QP_CREATE_DIR_FAILED + ' : ' + Dir, mtError, [mbOK], 0);
        Result := False;
      end;

  end;
end;

{-----------------------------------------------------------------------------}

// copy a directory tree
procedure DirCopy(FromDir, ToDir : string);
var
  FOS : TJvSHFileOperation;
begin
  FromDir := Trim(FromDir);
  if FromDir[Length(FromDir)] <> '\' then FromDir := FromDir + '\';
    ToDir := Trim(ToDir);
  FOS := TjvSHFileOperation.Create(nil);
  try
    FOS.Operation := FOCOPY;
    FOS.SourceFiles.Add(FromDir+'*.*');
    FOS.DestFiles.Add(ToDir);
    FOS.Options := [FOFALLOWUNDO, FOFMULTIDESTFILES, FOFNOCONFIRMMKDIR];
    FOS.Execute;
  Finally
    FreeAndNil(FOS);
  end;
end;

{-----------------------------------------------------------------------------}

//Function to get the type of EXE - WIN or DOS.  Wrapper for the JCL function
// GetFileExeType - and converts its result to a string.
function ExeTypeToString(const fName: string): String;
begin
  Result := '';
  Case GetFileExeType(fName) of
    etError : Result := QP_EXE_ERROR;
    etMSDos : Result := QP_EXE_DOS;
    etWin16 : Result := QP_EXE_WIN16;
    etWin32Gui,
    etWin32Con : Result := QP_EXE_WIN32;
  end;

end;

{-----------------------------------------------------------------------------}

procedure FileCopy(Source, Destination : String; DelSource : Boolean = false);
begin
  CopyFile(PChar(Source), PChar(Destination), False);

  if Delsource = true then
    SysUtils.DeleteFile(Source);
end;

{-----------------------------------------------------------------------------}

Function FileIsImage(InFile : TFileName) : Boolean;
const
  ImageExts : Array [0..11] of string = ('.png', '.gif', '.jpg', '.jpeg', '.jpe', '.jif', '.jfif', '.bmp', '.dib', '.rle', '.tiff', '.wmf');
var
  i : Integer;
begin
  Result := False;
  For i := 0 to 11 do
    if JCLStrings.StrCompare(ExtractFileExt(InFile), ImageExts[i]) = 0 then
    begin
      Result := True;
      break;
    end;
end;
{-----------------------------------------------------------------------------}

function GetEXEIcon(var myIcon : TIcon; filename : String) : Boolean;
var
  IconIndex: word;
  Buffer: array[0..2048] of char;
  IconHandle: HIcon;
begin

  Result := False;
  //if the file doesnt exist leave the iconindex at -1
  if not FileExists(filename) then
    exit;

  //get the associated icon for this file
  StrCopy(@Buffer, pchar(filename));
  IconIndex := 0;
  IconHandle := ExtractAssociatedIcon(HInstance, Buffer, IconIndex);

  if IconHandle = 0 then
    result := False
  else
  begin
    myIcon.Handle := IconHandle;
    Result := True;
  end;

end;

{-----------------------------------------------------------------------------}

function GetShortForm(Part : String) : String;
begin

  //Helper function for  GetVirtualShortName.
  //basically this function turns the parts of a file path into short form.

  Result := StrRemoveChars(part, [' ']);

  {$IFDEF DEBUG}
  AddToLog('After Removing spaces - ' + Result);
  {$ENDIF}

  If Length(Result) > 8 then
  begin
    Result := Copy(Result, 1, 6) + '~1' + ExtractFileExt(Part);
  end;

  {$IFDEF DEBUG}
  AddToLog('Making 8 characters long - ' + Result);
  {$ENDIF}

end;

{-----------------------------------------------------------------------------}

function GetVirtualShortName(FileName : TFileName) : String;
var
  Split : TStringList;
  Seperator : char;
  I : Integer;
begin

  //function to get the short name of a VIRTUAL file - ie a file that doesnt exist.

  Result := '';

  Split := TStringList.Create;
  Try

    //try to work out what seperator is used in this string.
    If CharPos(FileName, '/') <> 0 then
      Seperator := '/'
    else
      Seperator := '\';

    StrTokenToStrings(FileName, Seperator, Split);

    For i := 0 to Split.Count-1 do
      Result := Result + '\' + GetShortForm(Split[i]);

    //need to get rid of the starting '\'
    Result := Copy(Result, 2, Length(Result));

  Finally
    FreeAndNil(Split);
  end;
end;

{-----------------------------------------------------------------------------}

function HasSubDirs(const Folder: string): Boolean;
// Determines whether folder contains other file objects.
var
  SR: TSearchRec;
begin

  Result := FindFirst(IncludeTrailingPathDelimiter(Folder)+'*.*', faDirectory, SR) = 0;
  if Result then
    FindClose(SR);

end;

{-----------------------------------------------------------------------------}

function IsDriveReadOnly(Path : String) : Boolean;
var
  OldDir : String;
begin
  OldDir := GetCurrentDir();
  Try
    SetCurrentDir(Path);
    Result := (DiskFree(0) = 0);
  Finally
    SetCurrentDir(OldDir);
  end;
end;
{-----------------------------------------------------------------------------}


end.
